import { render, createEnhancer } from 'react-universal-interface';
import { Render } from '../Render';
import { h } from '../util';
import { easing } from './easing';
export const Tween = (props) => {
    let { easing: fn = 'linear' } = props;
    if (typeof props.easing === 'string') {
        fn = easing[fn];
    }
    if (process.env.NODE_ENV !== 'production') {
        if (typeof fn !== 'function') {
            console.error('<Tween> expected "easing" property to be a valid easing function or a 4-tuple array or numbers ' +
                'specifying a Cubic Bezier curve, or a string specifying one of the built-int easing functions: ' +
                '"' + Object.keys(easing).join('", "') + '".');
            console.trace();
            return null;
        }
    }
    return h(props.Render, props, ({ value }) => render(props, { value: fn(value) }));
};
Tween.defaultProps = {
    easing: 'linear',
    Render
};
export const withTween = createEnhancer(Tween, 'tween');
